
# q-expansion of newform 625.2.a.e, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 625
weight = 2
poly_data = [1, 16, -27, -35, 26, 20, -8, -3, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0], 1], [[-3, -1, 1, 0, 0, 0, 0, 0], 1], [[1, -15, -1, 17, -2, -4, 1, 0], 3], [[0, 4, -15, -1, 17, -2, -4, 1], 3], [[-4, 36, -5, -26, 5, 4, -1, 0], 3], [[0, -8, 51, -4, -43, 7, 8, -2], 3], [[-6, 1, 57, -7, -43, 7, 8, -2], 3]]

hecke_ring_character_values = None
aps_data = [[-1, 1, 0, 0, 0, 0, 0, 0], [0, 0, 0, 1, 0, -1, 0, 1], [0, 0, 0, 0, 0, 0, 0, 0], [-1, -1, 0, 0, 0, 1, 0, -1], [2, 1, 0, 1, -2, 0, 2, -1], [-3, 0, 0, -1, 1, 0, -2, 1], [-4, 0, -1, -3, 0, 1, -1, -1], [-2, -2, -1, -2, 1, 0, -1, -2], [-3, -1, 0, 0, -1, 1, 0, -1], [-2, 1, -1, -3, 1, 1, -2, -1], [1, 0, 2, 4, 0, -1, 1, 4], [2, -1, 2, 2, 1, 1, 1, 2], [-4, 2, -1, -3, 1, -2, -4, 1], [2, 0, 0, 1, -1, -2, 2, 1], [-2, 1, 0, 1, -1, -1, 1, -2], [-1, 3, 1, 2, -1, 1, 3, 3], [-3, 0, -1, -4, 1, -1, -2, -2], [-1, -1, -1, 0, 2, 3, 0, -1], [-1, 2, 1, 1, 1, 0, 4, 1], [-1, 2, 2, 2, 1, -1, 0, 2], [-2, -3, -1, -4, 3, -1, 1, 0], [0, -1, -1, 4, 0, 1, -1, -4], [-9, 1, -2, -1, 1, 3, -2, 2], [3, 0, 4, 3, -1, -3, 1, 3], [1, 0, -3, 4, 1, -2, -1, 1], [-4, 1, -1, 1, -2, 3, -1, 2], [-2, 0, 2, 1, 0, -2, 2, -3], [-2, -1, 1, 0, 0, -2, 1, -3], [0, 2, 2, 3, -3, 0, 6, 2], [0, 2, 2, -2, -3, -1, 3, -2], [1, 3, -2, -1, -4, 1, 1, 0], [3, -2, -3, 6, -1, -1, 2, 1], [9, 0, 0, 4, -1, -3, -2, -1], [-4, 2, -3, 0, 0, -2, -6, 2], [6, -6, 3, 0, 2, 1, 2, -3], [0, 2, 0, 3, 1, -5, 4, 3], [5, -4, -1, -1, 6, -1, -3, -3], [6, -5, 1, -1, 1, 2, -1, -6], [-10, -2, 0, 3, 4, -2, -3, 6], [2, 5, 5, 2, -3, -4, 4, 2], [-3, 2, 0, 4, -1, -2, 3, 10], [-4, -5, 0, -7, 5, 0, 0, -5], [4, -1, 2, 9, -3, -1, -1, 0], [4, 6, -2, -2, -3, 4, -1, -2], [6, -2, 2, 5, -5, 3, 1, -10], [0, 0, -3, 6, 0, -1, -4, 4], [-3, -4, 4, 3, 3, -2, 4, 2], [-4, 4, -4, 5, -1, -2, 1, 7], [-20, -2, -1, -11, 6, 3, -8, -2], [6, -2, 0, -2, -8, 3, 0, -6], [5, -5, -1, -3, -2, 0, 5, -3], [2, -4, -1, -8, -3, 2, -2, -3], [-2, 5, 0, 2, 0, 0, 5, 10], [5, -3, -3, -1, 6, -1, 0, -3], [1, 1, 3, 4, 1, 2, 11, 4], [-9, -5, 1, -1, 6, -3, -1, -1], [-6, 7, 1, -5, -2, 4, -7, 3], [-4, -5, 2, -1, 5, 4, 1, 4], [6, 4, 3, 11, -1, -4, 5, 3], [-10, 1, -2, -9, -2, 1, 6, 0], [9, 1, 5, 14, -4, 2, 10, 3], [7, -6, 0, 9, -4, -2, -2, -2], [-14, 4, -2, -15, 0, 2, -7, -7], [-1, -6, 1, -4, 2, 2, 1, -12], [7, 3, 5, 5, 0, -5, 0, 15], [8, -5, 7, -3, 0, -3, -3, 3], [-4, 5, 1, 1, 0, -3, -2, 12], [10, -4, 4, -4, -3, -5, 1, -1], [-1, 3, 4, -5, -5, -3, 3, 1], [-12, -3, -1, -15, -4, 3, -9, -9], [-6, 1, 1, -7, -2, 1, -10, 2], [-3, 5, -2, -2, -4, 2, -5, 1], [10, 2, 0, 11, -9, 3, 8, -2], [-6, -4, -2, -16, 0, 2, -4, -11], [4, 5, -1, -2, -2, 6, -4, 0], [-7, 3, 1, 3, -3, 4, -1, 2], [-11, -6, -6, -6, 6, 7, -6, -11], [-6, -6, -6, -8, 3, 7, -3, -1], [9, -2, 2, 8, -1, -6, -3, 11], [0, 2, 3, 6, 7, 2, -1, 9], [9, -8, -6, -1, 3, 5, -8, -1], [4, -3, 9, 13, 1, -2, 11, 1], [-7, -2, 2, -9, -1, 9, -8, -9], [5, 2, -1, 6, -1, 0, 3, -7], [12, 6, -1, -3, 2, -3, 3, -3], [-14, 8, 1, -3, -2, 0, -10, 3], [7, -2, -6, -1, -1, -7, -2, 0], [8, -1, -9, 10, -5, 8, -4, -3], [7, 6, 4, 18, -7, -2, 4, 7], [21, 4, 1, 12, -7, 2, 10, 1], [-10, 5, 0, -8, 0, -2, 0, -3], [-15, -6, -2, -9, 5, 9, 1, -6], [0, 6, -1, 1, -3, -5, 6, -6], [6, -3, 3, 4, -4, 1, 8, -1], [-2, -3, -5, 3, 6, 0, 4, -7], [-3, -1, -2, 4, -8, 0, 9, -8], [8, -11, -5, 0, 8, -4, -6, 6], [3, 8, 0, 5, -1, -5, 6, -3], [9, 2, -2, 6, -7, -3, -10, -3], [5, 8, 2, 5, -6, -1, 12, -9], [1, -2, -3, -10, 5, -2, -4, 0], [6, 6, -2, -6, -4, -3, 1, 9], [7, -3, 0, 18, -3, 0, 11, -8], [-11, -3, 4, -11, 3, -5, -3, 9], [-23, 4, -5, -26, 7, 10, -17, -4], [12, -9, 2, -5, 5, 4, 7, -4], [-6, -1, 0, -13, -4, 7, -6, -9], [-4, 2, 1, 10, 10, 0, 1, 15], [4, 6, 8, 3, -2, 6, 16, 10], [1, 11, 6, 7, 3, -8, 0, 21], [-3, 1, 7, -1, 6, 0, -10, 6], [9, 0, 3, 4, -9, -4, -4, 0], [-13, 2, -12, -24, 1, 4, -19, -8], [-17, 13, -5, 1, 1, 7, -3, 10], [28, 3, 1, 8, -6, -3, 4, -1], [-8, 11, -6, -2, -7, -2, -1, 2], [4, -4, -2, -2, -3, 9, -6, -12], [-37, -6, -10, -18, 13, -1, -22, 2], [6, 4, 9, 4, -3, 2, -5, 0], [-3, -5, -3, -5, -4, 5, 2, -21], [-5, 3, -5, 19, -1, -5, 1, 7], [-19, 14, -9, -15, 4, 3, -9, 2], [5, -6, -1, 16, -6, -2, 11, 1], [8, 0, -4, 22, -2, -6, -3, 13], [-3, 11, -1, 8, 3, -2, -1, 7], [-5, -3, 4, 5, 1, 8, 6, 11], [2, 5, 0, 9, 1, -9, -9, 5], [-15, 1, -2, 2, 5, 8, 2, -7], [-2, 12, -4, 7, 3, 2, -5, 1], [-4, -7, 3, 3, 7, 3, 6, -7], [-27, 0, -3, -13, 4, 3, -5, 4], [-21, -1, -12, -16, 6, 4, -4, -3], [14, 8, 0, 2, -6, 5, 6, -13], [-3, -2, 7, 3, -3, 0, -7, 2], [9, -16, -8, -1, 7, -6, -6, 0], [4, 2, 6, 10, -7, -6, -2, 3], [15, -2, 7, 18, -4, 5, 8, -8], [-2, -3, 1, 8, 0, -1, -3, 18], [-7, 15, 0, -6, -9, -6, 7, 5], [-2, -1, 11, -2, -2, -7, 7, 8], [-15, 3, -6, -14, 4, -2, -7, 0], [1, 0, 3, -8, -5, -9, 4, -9], [-1, -12, 8, -7, 11, -3, 4, 0], [-6, 9, 0, -9, -6, 0, 12, 9], [9, 0, 4, 8, -2, -4, 6, 10], [-7, -5, -4, -1, 4, -9, -13, 7], [3, -4, -8, -17, 8, 3, -4, -10], [3, -3, -4, 11, 4, -7, 13, 1], [17, -10, 4, 1, 1, 4, 8, -2], [-19, 3, 1, -5, 5, 2, 1, -2], [6, 1, 8, 15, -10, -9, 8, 14], [-24, 0, -8, 4, 10, -1, -14, -1], [15, -10, -1, 12, 3, 0, 0, -13], [-23, 15, 2, -11, -11, 6, 10, 0], [2, -13, -3, -9, 9, 0, 4, -11], [26, -8, 1, 3, -4, 2, -3, -15], [-4, 0, -1, -4, 7, -10, -7, 1], [13, 6, 11, 13, -4, -5, 19, 3], [2, 3, 7, 10, -7, -9, 7, 4], [14, -6, -2, -8, 2, 1, -3, 7], [6, -6, 7, 7, 8, -7, -15, 23], [-9, -14, -11, -2, 13, -3, -17, -9], [-17, 5, -3, 7, 5, 2, -3, 18], [10, 11, 0, -13, -7, 10, 2, -11], [-7, 2, 1, -13, -7, 2, -10, -4], [-10, 3, 9, -12, 2, -5, -3, -4], [6, -3, 5, 0, -4, 0, -6, -12], [-1, 10, -1, 14, 1, -3, -14, 7]]
